/* -*- Mode: c++; -*- */
/*  --------------------------------------------------------------------
 *  Filename:
 *    src/mailbox/maildir-open.cc
 *  
 *  Description:
 *    Implementation of the Maildir class.
 *
 *  Authors:
 *    Andreas Aardal Hanssen <bincimap@andreas.hanssen.name>
 *
 *  Bugs:
 *
 *  ChangeLog:
 *
 *  --------------------------------------------------------------------
 *  Copyright 2002, 2003 Andreas Aardal Hanssen
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Street #330, Boston, MA 02111-1307, USA.
 *  --------------------------------------------------------------------
 */
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <stdio.h>

// Required for POSIX.1 open() and O_RDONLY flags
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

#include "maildir.h"

using namespace ::std;

//------------------------------------------------------------------------
// open returns a posix file descriptor for the given file name
// Added 20030228 --cld
int Binc::Maildir::open(const string &fixed)
{
  int fp;
  string actualfilename = fixedtofile[fixed];
  if (actualfilename == "")
    return -1;

  while ((fp = ::open((path + "/cur/" + actualfilename).c_str(), O_RDONLY)) == -1) {
    scanFileNames();
    if (fixedtofile.find(fixed) == fixedtofile.end())
      return -1;
    actualfilename = fixedtofile[fixed];
  }

  return fp;
}

//------------------------------------------------------------------------
// fopen returns a C stream FILE* for the given file name
FILE *Binc::Maildir::fopen(const string &fixed) const
{
  FILE *fp;
  string actualfilename = fixedtofile[fixed];
  if (actualfilename == "")
    return NULL;

  while ((fp = ::fopen((path + "/cur/" + actualfilename).c_str(), "r"))
      == NULL) {
    scanFileNames();
    if (fixedtofile.find(fixed) == fixedtofile.end())
      return NULL;
    actualfilename = fixedtofile[fixed];
  }

  return fp;
}

