/* -*- Mode: c++; -*- */
/*  --------------------------------------------------------------------
 *  Filename:
 *    src/mailbox/maildir-scanfilesnames.cc
 *  
 *  Description:
 *    Implementation of the Maildir class.
 *
 *  Authors:
 *    Andreas Aardal Hanssen <bincimap@andreas.hanssen.name>
 *
 *  Bugs:
 *
 *  ChangeLog:
 *
 *  --------------------------------------------------------------------
 *  Copyright 2002, 2003 Andreas Aardal Hanssen
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Street #330, Boston, MA 02111-1307, USA.
 *  --------------------------------------------------------------------
 */
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "maildir.h"

#include <iostream>
#include <iomanip>

#include <dirent.h>
#include <unistd.h>
#include <errno.h>

#include "../../io/io.h"

using namespace ::std;

//------------------------------------------------------------------------
void Binc::Maildir::scanFileNames(void) const
{
  string curpath = path + "/cur/";
  DIR *pdir = opendir(curpath.c_str());
  if (pdir == NULL) {
    string reason = "Maildir::scan::opendir(\"" + curpath + "\") == NULL (";
    reason += strerror(errno);
    reason += ")";

    throw MessageException(reason);
  }

  fixedtofile.clear();

  struct dirent *pdirent;
  while ((pdirent = readdir(pdir)) != NULL) {
    if (!isdigit(pdirent->d_name[0])) continue;

    string filename = pdirent->d_name;
    string time;
    string hostname;
    string nrepeat_count;
    
    string::size_type pos = filename.find('.');
    if (pos == string::npos) continue;
    
    time = filename.substr(0, pos);
    filename = filename.substr(pos + 1);
    
    if ((pos = filename.find('.')) == string::npos) continue;
    
    nrepeat_count = filename.substr(0, pos);
    filename = filename.substr(pos + 1);    
    
    
    if ((pos = filename.find(':')) == string::npos)
      hostname = filename;
    else
      hostname = filename.substr(0, pos);

    BincStream ss;
     
    ss << setw(10) << setfill('0') << atoi(time.c_str());

    string fixedfilename = ss.str();
    fixedfilename += "." + nrepeat_count + "." + hostname;

    fixedtofile[fixedfilename] = pdirent->d_name;
  }

  closedir(pdir);
}
