/* -*- Mode: c++; -*- */
/*  --------------------------------------------------------------------
 *  Filename:
 *    src/mailbox/maildir-updateflags.cc
 *  
 *  Description:
 *    Implementation of the Maildir class.
 *
 *  Authors:
 *    Andreas Aardal Hanssen <bincimap@andreas.hanssen.name>
 *
 *  Bugs:
 *
 *  ChangeLog:
 *
 *  --------------------------------------------------------------------
 *  Copyright 2002, 2003 Andreas Aardal Hanssen
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Street #330, Boston, MA 02111-1307, USA.
 *  --------------------------------------------------------------------
 */
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "maildir.h"

#include <dirent.h>
#include <unistd.h>

#include "../../io/io.h"

using namespace ::std;

//------------------------------------------------------------------------
void Binc::Maildir::updateFlags(void)
{
  IO &logger = IOFactory::getInstance().get(2);

  if (readonly) return;

  string curpath = path + "/cur/";
  DIR *pdir = opendir(curpath.c_str());
  if (pdir == NULL) {
    string reason = "Maildir::scan::opendir(\"" + curpath + "\") == NULL (";
    reason += strerror(errno);
    reason += ")";
    
    throw MessageException(reason);
  }
  
  struct dirent *pdirent;
  while ((pdirent = readdir(pdir)) != NULL) {
    if (!isdigit(pdirent->d_name[0])) continue;
    
    string filename = pdirent->d_name;
    string time;
    string hostname;
    string nrepeat_count;
    string info;
    
    string::size_type pos = filename.find('.');
    if (pos == string::npos) continue;
    
    time = filename.substr(0, pos);
    filename = filename.substr(pos + 1);
    
    if ((pos = filename.find('.')) == string::npos) continue;
    
    nrepeat_count = filename.substr(0, pos);
    filename = filename.substr(pos + 1);    
    
    
    if ((pos = filename.find(':')) == string::npos)
      hostname = filename;
    else {
      hostname = filename.substr(0, pos);
      info = filename.substr(pos + 1);    
    }
    
    string experimental;
    string standard;
    if (info != "")
      if ((pos = info.find("2,")) != string::npos) {
	standard = info.substr(pos + 2);
      }
    
    Message m;

    for (string::const_iterator i = standard.begin(); 
	 i != standard.end(); ++i)
      switch (*i) {
      case 'R': m.setStdFlag(Message::F_ANSWERED); break;
      case 'S': m.setStdFlag(Message::F_SEEN); break;
      case 'T': m.setStdFlag(Message::F_DELETED); break;
      case 'D': m.setStdFlag(Message::F_DRAFT); break;
      case 'F': m.setStdFlag(Message::F_FLAGGED); break;
      default: continue;
      }
    
    BincStream ss;

    ss << setw(10) << setfill('0') << atoi(time.c_str());

    //    string fixedfilename = time + "." + nrepeat_count + "." + hostname;
    string fixedfilename = ss.str();
    fixedfilename += "." + nrepeat_count + "." + hostname;

    // if we already have details about this file in memory..
    if (has(fixedfilename)) {

      Message &oldm = get(fixedfilename);
      if (m.getStdFlags() == oldm.getStdFlags()) {
	// the flags are equal, let's skip to the next
	continue;
      } else {
	// the flags are different. we need to report this
	string newinfo = ":2,";
	int flags = oldm.getStdFlags();
	if (flags & Message::F_DRAFT) newinfo += "D";
	if (flags & Message::F_FLAGGED) newinfo += "F";
	if (flags & Message::F_ANSWERED) newinfo += "R";
	if (flags & Message::F_SEEN) newinfo += "S";
	if (flags & Message::F_DELETED) newinfo += "T";

	// return value is deliberately ignored. if the rename failed,
	// it means that another client has changed the flags. we will
	// respect that.
	string destname = curpath + static_cast<string>(time) 
	  + "." + nrepeat_count 
	  + "." + hostname + newinfo;
	string srcname = (curpath + pdirent->d_name);

	if (srcname != destname) {
	  if (link(srcname.c_str(), destname.c_str()) != 0) {
	    logger << "warning: link(" << (curpath + pdirent->d_name)
		   << "," << ((curpath + static_cast<string>(time)
			       + "." + nrepeat_count 
			       + "." + hostname + newinfo).c_str()) << ") == "
		   << errno << ": " << strerror(errno) << endl;
	  } else if (unlink((curpath + pdirent->d_name).c_str()) != 0) {
	    logger << "warning: unlink(" << (curpath + pdirent->d_name) << ") == "
		   << errno << ": " << strerror(errno) << endl;	  
	  }
	}

	continue;
      }
    }
  }

  closedir(pdir);
}
