/* -*- Mode: c++; -*- */
/*  --------------------------------------------------------------------
 *  Filename:
 *    src/mailbox/message.cc
 *  
 *  Description:
 *    Implementation of the Message class.
 *
 *  Authors:
 *    Andreas Aardal Hanssen <bincimap@andreas.hanssen.name>
 *
 *  Bugs:
 *
 *  ChangeLog:
 *
 *  --------------------------------------------------------------------
 *  Copyright 2002, 2003 Andreas Aardal Hanssen
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Street #330, Boston, MA 02111-1307, USA.
 *  --------------------------------------------------------------------
 */
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <string>

#include <ctype.h>
#include <time.h>

#include "message.h"
#include "../util/convert.h"
#include "../parsers/mime/mime.h"
#include "../io/io.h"

using namespace ::std;

//------------------------------------------------------------------------
Binc::Message::Message(void) : MimeDocument(), tolist(), cclist(), bcclist()
{
  expunged = false;
  stdflags = F_NONE;
  flagsChanged = false;
}

//------------------------------------------------------------------------
Binc::Message::~Message(void)
{
}

//------------------------------------------------------------------------
void Binc::Message::close(void)
{
  clear();
}

//------------------------------------------------------------------------
void Binc::Message::setExpunged(void)
{
  expunged = true;
}

//------------------------------------------------------------------------
void Binc::Message::setUnExpunged(void)
{
  expunged = false;
}

//------------------------------------------------------------------------
void Binc::Message::setFlagsUnchanged(void)
{
  flagsChanged = false;
}

//------------------------------------------------------------------------
bool Binc::Message::hasFlagsChanged(void)
{
  return flagsChanged;
}

//------------------------------------------------------------------------
int Binc::Message::getStdFlags(void) const
{
  return stdflags;
}

//------------------------------------------------------------------------
bool Binc::Message::isExpunged(void) const
{
  return expunged;
}

//------------------------------------------------------------------------
int Binc::Message::getUID(void) const
{
  return uid;
}

//------------------------------------------------------------------------
int Binc::Message::getSize(void) const
{
  return size;
}

//------------------------------------------------------------------------
const string &Binc::Message::getID(void) const
{
  return id;
}

//------------------------------------------------------------------------
const string Binc::Message::getInternalDate(void) const
{
  time_t _t = atoi(id.c_str());
  struct tm *_tm = gmtime(&_t);
  char internal[64];
  if (strftime(internal, sizeof(internal), "%d-%b-%Y %H:%M:%S %Z", _tm) == 0)
    return "NIL";

  return internal;
}

//------------------------------------------------------------------------
void Binc::Message::setStdFlag(int f_in)
{
  flagsChanged = true;
  stdflags |= f_in;
}

//------------------------------------------------------------------------
void Binc::Message::resetStdFlags(void)
{
  flagsChanged = true;
  stdflags = F_NONE;
}

//------------------------------------------------------------------------
void Binc::Message::setUID(int i_in)
{
  uid = i_in;
}

//------------------------------------------------------------------------
void Binc::Message::setSize(int i_in)
{
  size = i_in;
}

//------------------------------------------------------------------------
void Binc::Message::setID(const string &s_in)
{
  id = s_in;
}
