/* -*- Mode: c++; -*- */
/*  --------------------------------------------------------------------
 *  Filename:
 *    src/mailbox/message.h
 *  
 *  Description:
 *    Declaration of the Message class.
 *
 *  Authors:
 *    Andreas Aardal Hanssen <bincimap@andreas.hanssen.name>
 *
 *  Bugs:
 *
 *  ChangeLog:
 *
 *  --------------------------------------------------------------------
 *  Copyright 2002, 2003 Andreas Aardal Hanssen
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Street #330, Boston, MA 02111-1307, USA.
 *  --------------------------------------------------------------------
 */
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#ifndef message_h_included
#define message_h_included
#include <string>
#include <map>
#include <vector>
#include <exception>
#include <iostream>
#include <time.h>

#include <stdio.h>
#include <string.h>

#include "address.h"
#include "../parsers/mime/mime.h"

namespace Binc {

  //------------------------------------------------------------------------
  class Message : public Binc::MimeDocument {
    int stdflags;
    int uid;
    bool expunged;
    bool flagsChanged;
    std::string id;

    mutable bool mimeParsed;
    
  public:

    void parseMime(void) const;

    mutable std::vector<Address> tolist;
    mutable std::vector<Address> cclist;
    mutable std::vector<Address> bcclist;

    enum {
      F_NONE = 0x00,
      F_SEEN = 0x01,
      F_ANSWERED = 0x02,
      F_DELETED = 0x04,
      F_DRAFT = 0x08,
      F_RECENT = 0x10,
      F_FLAGGED = 0x20,
      F_EXPUNGED = 0x40,
      F_JUSTARRIVED = 0x80
    };

    //-- access methods, get

    int getUID(void) const;
    int getSize(void) const;
    int getStdFlags(void) const;

    const std::string &getID(void) const;
    const std::string getInternalDate(void) const;
    //    const std::string getFixedFilename(void) const;

    bool isExpunged(void) const;

    //-- access methods, set

    void setExpunged(void);
    void setUnExpunged(void);

    void setID(const std::string &s_in);

    void setStdFlag(int f_in);
    void resetStdFlags(void);
    void setUID(int i_in);
    void setSize(int s);
    void setFlagsUnchanged(void);
    bool hasFlagsChanged(void);

    void setUserDefFlag(const std::string &f_in);

    void close(void);

    // construction & destruction

    Message(void);
    ~Message(void);
  };

}

#endif
