/* -*- Mode: c++; -*- */
/*  --------------------------------------------------------------------
 *  Filename:
 *    src/operator-logout.cc
 *  
 *  Description:
 *    Implementation of the LOGOUT command
 *
 *  Authors:
 *    Andreas Aardal Hanssen <bincimap@andreas.hanssen.name>
 *
 *  Bugs:
 *
 *  ChangeLog:
 *
 *  --------------------------------------------------------------------
 *  Copyright 2002, 2003 Andreas Aardal Hanssen
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Street #330, Boston, MA 02111-1307, USA.
 *  --------------------------------------------------------------------
 */
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <string>
#include <iostream>

#include "io/io.h"

#include "mailbox/mailbox.h"
#include "parsers/imap/recursivedescent/recursivedescent.h"
#include "session.h"
#include "util/convert.h"

#include "depot.h"
#include "operators.h"

using namespace ::std;
using namespace Binc;

//----------------------------------------------------------------------
LogoutOperator::LogoutOperator(void)
{
}

//----------------------------------------------------------------------
LogoutOperator::~LogoutOperator(void)
{
}

//----------------------------------------------------------------------
const string LogoutOperator::getName(void) const
{
  return "LOGOUT";
}

//----------------------------------------------------------------------
int LogoutOperator::getState(void) const
{
  return Session::NONAUTHENTICATED | Session::AUTHENTICATED | Session::SELECTED;
}

//------------------------------------------------------------------------
void LogoutOperator::process(Depot &depot, Command &command) const
{
  IO &com = IOFactory::getInstance().get(1);

  com << "* BYE Binc IMAP shutting down" << endl;
  com << command.getTag() << " OK LOGOUT completed" << endl;
  com.flushContent();
  
  Mailbox *mailbox = NULL;
  if ((mailbox = depot.getSelected()) != NULL) {
    mailbox->imapClose();
    delete mailbox;
  }
  
  Session &session = Session::getInstance();
  session.setState(Session::LOGOUT);

  throw OperatorException(Operator::NOTHING, "");
}

//----------------------------------------------------------------------
bool LogoutOperator::parse(Command & c_in) const {
  try {
    if (!expectThisString("LOGOUT"))
      return false;
    if (!expectCRLF())
      throw SyntaxException("expected CRLF");
  } catch (InputException &e) {
    e.addReason("in LogoutOperator::parse");
    throw e;
  }

  c_in.setName("LOGOUT");
  return true;
}
