/* -*- Mode: c++; -*- */
/*  --------------------------------------------------------------------
 *  Filename:
 *    src/bincimapd-lsub.cc
 *  
 *  Description:
 *    Implementation of the LSUB command.
 *
 *  Authors:
 *    Andreas Aardal Hanssen <bincimap@andreas.hanssen.name>
 *
 *  Bugs:
 *
 *  ChangeLog:
 *
 *  --------------------------------------------------------------------
 *  Copyright 2002, 2003 Andreas Aardal Hanssen
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Street #330, Boston, MA 02111-1307, USA.
 *  --------------------------------------------------------------------
 */
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <algorithm>
#include <string>
#include <vector>
#include <iostream>

#include "parsers/imap/recursivedescent/recursivedescent.h"

#include "io/io.h"
#include "util/convert.h"

#include "util/regmatch.h"

#include "session.h"
#include "depot.h"
#include "operators.h"

using namespace ::std;
using namespace Binc;

//----------------------------------------------------------------------
LsubOperator::LsubOperator(void)
{
}

//----------------------------------------------------------------------
LsubOperator::~LsubOperator(void)
{
}

//----------------------------------------------------------------------
const string LsubOperator::getName(void) const
{
  return "LSUB";
}

//----------------------------------------------------------------------
int LsubOperator::getState(void) const
{
  return Session::AUTHENTICATED | Session::SELECTED;
}

//------------------------------------------------------------------------
void LsubOperator::process(Depot &depot, Command &command) const
{
  Session &session = Session::getInstance();
  IO &com = IOFactory::getInstance().get(1);

  string wildcard = command.getListMailbox();
  if (wildcard != "" && wildcard[0] == depot.getDelimiter()) 
    wildcard = wildcard.substr(1);

  const string &regex = toRegex(wildcard, depot.getDelimiter());
  string ref = command.getMailbox();
  if (ref != "" && ref[0] == depot.getDelimiter()) 
    ref = ref.substr(1);

  vector<string> &subscribed = session.subscribed;
  sort(subscribed.begin(), subscribed.end());

  for (vector<string>::const_iterator i = subscribed.begin();
       i != subscribed.end(); ++i) {
    if (ref == "" || ((ref.length() < (*i).length()) 
		      && ((*i).substr(0, ref.length()) == ref))) 
      if (regexMatch(*i, regex) == 0)
	com << "*" << " LSUB () \"" << depot.getDelimiter() << "\" "
	    << toImapString(*i) << endl; 
  }
}

//----------------------------------------------------------------------
bool LsubOperator::parse(Command &c_in) const
{
  try {
    if (!expectThisString("LSUB"))
      return false;

    if (!expectSPACE())
      throw SyntaxException("expected SPACE");

    string mailbox;
    if (!expectMailbox(mailbox))
      throw SyntaxException("expected mailbox");
    c_in.setMailbox(mailbox);

    if (!expectSPACE())
      throw SyntaxException("expected SPACE");

    string listmailbox;
    if (!expectListMailbox(listmailbox))
      throw SyntaxException("expected list_mailbox");
    if (!expectCRLF())
      throw SyntaxException("expected CRLF");
    c_in.setListMailbox(listmailbox);
  } catch (InputException &e) {
    e.addReason("in LsubOperator::parse");
    throw e;
  } catch (SyntaxException &e) {
    e.addReason("in LsubOperator::parse");
    throw e;
  }

  c_in.setName("LSUB");
  return true;
}
