/* -*- Mode: c++; -*- */
/*  --------------------------------------------------------------------
 *  Filename:
 *    src/operator-noop.cc
 *  
 *  Description:
 *    Operator for the NOOP command.
 *
 *  Authors:
 *    Andreas Aardal Hanssen <bincimap@andreas.hanssen.name>
 *
 *  Bugs:
 *
 *  ChangeLog:
 *
 *  --------------------------------------------------------------------
 *  Copyright 2002, 2003 Andreas Aardal Hanssen
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Street #330, Boston, MA 02111-1307, USA.
 *  --------------------------------------------------------------------
 */
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <string>
#include <iostream>

#include "io/io.h"

#include "parsers/imap/recursivedescent/recursivedescent.h"
#include "session.h"
#include "depot.h"
#include "operators.h"

using namespace ::std;
using namespace Binc;

//----------------------------------------------------------------------
NoopOperator::NoopOperator(void)
{
}

//----------------------------------------------------------------------
NoopOperator::~NoopOperator(void)
{
}

//----------------------------------------------------------------------
const string NoopOperator::getName(void) const
{
  return "NOOP";
}

//----------------------------------------------------------------------
int NoopOperator::getState(void) const
{
  return Session::NONAUTHENTICATED | Session::AUTHENTICATED | Session::SELECTED;
}

//----------------------------------------------------------------------
void NoopOperator::process(Depot &depot, Command &command) const
{
}

//----------------------------------------------------------------------
bool NoopOperator::parse(Command &c_in) const
{
  try {
    if (!expectThisString("NOOP"))
      return false;
    if (!expectCRLF())
      throw SyntaxException("expected CRLF after NOOP");
  } catch (InputException &e) {
    e.addReason("in NoopOperator::parse");
    throw e;
  }

  c_in.setName("NOOP");
  return true;
}
