/* -*- Mode: c++; -*- */
/*  --------------------------------------------------------------------
 *  Filename:
 *    src/operator-starttls.cc
 *  
 *  Description:
 *    Implementation of the STARTTLS command.
 *
 *  Authors:
 *    Andreas Aardal Hanssen <bincimap@andreas.hanssen.name>
 *
 *  Bugs:
 *
 *  ChangeLog:
 *
 *  --------------------------------------------------------------------
 *  Copyright 2002, 2003 Andreas Aardal Hanssen
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Street #330, Boston, MA 02111-1307, USA.
 *  --------------------------------------------------------------------
 */
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#ifdef WITH_SSL

#include <string>
#include <iostream>

#include "parsers/imap/recursivedescent/recursivedescent.h"
#include "io/io.h"
#include "io/io-ssl.h"
#include "session.h"
#include "depot.h"
#include "operators.h"

using namespace ::std;
using namespace Binc;

//----------------------------------------------------------------------
StarttlsOperator::StarttlsOperator(void)
{
}

//----------------------------------------------------------------------
StarttlsOperator::~StarttlsOperator(void)
{
}

//----------------------------------------------------------------------
const string StarttlsOperator::getName(void) const
{
  return "STARTTLS";
}

//----------------------------------------------------------------------
int StarttlsOperator::getState(void) const
{
  return Session::NONAUTHENTICATED | Session::AUTHENTICATED | Session::SELECTED;
}

//------------------------------------------------------------------------
void StarttlsOperator::process(Depot &depot, Command &command) const
{
  Session &session = Session::getInstance();
  IO &com = IOFactory::getInstance().get(1);
  IO &logger = IOFactory::getInstance().get(2);

  if (session["sslmode"] != "")
    throw OperatorException(Operator::BAD, "already in TLS mode");

  session.globalconfig.setSection("Authentication");
  if (session.globalconfig["disable starttls"] == "yes")
    throw OperatorException(Operator::NO, "disabled by administrator");

  com << command.getTag() << " OK STARTTLS completed, begin TLS negotiation now" << endl;
  com.flushContent();
  com.flushOnEndl();
   
  try {
    SSLEnabledIO *sslcom = dynamic_cast<SSLEnabledIO *>(&IOFactory::getInstance().get(1));
    sslcom->setModeSSL();
  } catch (InputException &e) {
    logger << "TLS negotiation failed." << endl;
  }

  session.add("sslmode", "yes");

  throw OperatorException(Operator::NOTHING, "");
}

//----------------------------------------------------------------------
bool StarttlsOperator::parse(Command &c_in) const
{
  try {
    if (!expectThisString("STARTTLS"))
      return false;
    if (!expectCRLF())
      throw SyntaxException("expected CRLF");
  } catch (InputException &e) {
    e.addReason("in expectSTARTTLS");
    throw e;
  }

  c_in.setName("STARTTLS");
  return true;
}

#endif
