/* -*- Mode: c++; -*- */
/*  --------------------------------------------------------------------
 *  Filename:
 *    arg-exceptions.h
 *  
 *  Description:
 *    Exception classes for Argument Parser
 *
 *  Authors:
 *    Eivind Kvedalen <argparser@eivind.kvedalen.name>
 *
 *  Bugs:
 *
 *  ChangeLog:
 *
 *  --------------------------------------------------------------------
 *  Copyright 2002, 2003 Eivind Kvedalen
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Street #330, Boston, MA 02111-1307, USA.
 *  --------------------------------------------------------------------
 */

#ifndef _ARG_EXCEPTIONS_H
#define _ARG_EXCEPTIONS_H

#include <string>
#include <exception>
#include <set>

namespace ArgParser {

  class ArgumentException : public std::exception {
    std::string argName;
  public:
    ArgumentException(const std::string &n) : argName(n) { }
    virtual ~ArgumentException(void) throw () { }
    const std::string & name(void) const { return argName; }
  };
  
  class CheckException : public ArgumentException {
  public:
    CheckException(const std::string &n,
		   const std::string &error) : ArgumentException(n) { }
    virtual ~CheckException(void) throw() { }
  };

  class RequiredCommandException : public ArgumentException {
  public:
    RequiredCommandException() : ArgumentException("unknown") { }
    virtual  ~RequiredCommandException(void) throw() { }
  };

  class RequiredArgumentException : public ArgumentException {
  public:
    std::set<std::string> missing;
    RequiredArgumentException() : ArgumentException("unknown") { }
    virtual ~RequiredArgumentException(void) throw() { }
    void add(const std::string & s) {
      missing.insert(s);
    }
  };
  
  class UnknownArgumentException : public ArgumentException {
  public:
    UnknownArgumentException(const std::string &n) : ArgumentException(n) { }
    virtual ~UnknownArgumentException(void) throw() { }
  };
  
  class MissingArgumentException : public ArgumentException {
    std::string argName;
  public:
    MissingArgumentException(const std::string &n) : ArgumentException(n) { }
    virtual ~MissingArgumentException(void) throw() { }
  };
  
  class InvalidArgumentException : public ArgumentException {
  public:
    InvalidArgumentException(const std::string &n) : ArgumentException(n) { }
    virtual ~InvalidArgumentException(void) throw() { }
  };
  
  class MissingValueException : public ArgumentException {
  public:
    MissingValueException(const std::string &n) : ArgumentException(n) { }
    virtual ~MissingValueException(void) throw() { }
  };

}

#endif
