/* -*- Mode: c++; -*- */
/*  --------------------------------------------------------------------
 *  Filename:
 *    src/parsers/mime/mime.cc
 *  
 *  Description:
 *    Implementation of main mime parser components
 *
 *  Authors:
 *    Andreas Aardal Hanssen <bincimap@andreas.hanssen.name>
 *
 *  Bugs:
 *
 *  ChangeLog:
 *
 *  --------------------------------------------------------------------
 *  Copyright 2002, 2003 Andreas Aardal Hanssen
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Street #330, Boston, MA 02111-1307, USA.
 *  --------------------------------------------------------------------
 */
#ifndef mime_utils_h_included
#define mime_utils_h_included

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif


#include "mime.h"
#include "../../util/convert.h"
#include "../../io/io.h"
#include <string>
#include <vector>
#include <map>
#include <exception>
#include <iostream>

#include <string.h>
#include <ctype.h>
#include <stdio.h>
#include <errno.h>

using namespace ::std;

namespace {

  inline bool compareStringToQueue(const std::string &s_in, char *bqueue, int pos, int size)
  {
    if (s_in[0] != bqueue[pos % size]) return false;

    for (int i = 0; i < size; ++i)
      if (s_in.at(i) != bqueue[(pos + i) % size])
	return false;

    return true;
  }

  static stack<char> CRLFbuffer;
  static char CRLFlastchar;
  static int CRLFbuffersize;

  static char ringbuffer[8192];
  static char *pointer;
  static int buffersize;

  int fillBuffer(FILE *fp)
  {
    if ((buffersize = fread(ringbuffer, 1, sizeof(ringbuffer), fp)) == 0)
      return EOF;

    pointer = ringbuffer;
    return 0;
  }

  // read one character from the input file. pass the data as a pure
  // CRLF text file, regardless of what the source is.
  inline int fgetcCRLF(FILE *fp)
  {
    static int c;

    if (CRLFbuffer.size() != 0) {
      c = CRLFbuffer.top();
      CRLFbuffer.pop();
      CRLFlastchar = c;
      CRLFbuffersize--;
      return c;
    }

    while ((c = fgetc(fp)) != '\n')
      if (c != '\r')
	return CRLFlastchar = c;

    if (CRLFlastchar != '\r') {
      CRLFbuffer.push('\n');
      CRLFbuffersize++;
      return CRLFlastchar = '\r';
    }

    return CRLFlastchar = '\n';
  }

  inline void fungetcCRLF(char c)
  {
    CRLFbuffer.push(c);
    CRLFbuffersize++;
  }
}

#endif
