/* -*- Mode: yacc; -*- */
/*  --------------------------------------------------------------------
 *  Filename:
 *    src/parsers/storage/storageparser.y
 *  
 *  Description:
 *    Definition of the Storage grammar, in yacc
 *
 *  Authors:
 *    Andreas Aardal Hanssen <bincimap@andreas.hanssen.name>
 *
 *  Bugs:
 *
 *  ChangeLog:
 *
 *  --------------------------------------------------------------------
 *  Copyright 2002 Andreas Aardal Hanssen
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Street #330, Boston, MA 02111-1307, USA.
 *  --------------------------------------------------------------------
 */
%{
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <string>
#include <map>
#include <vector>
#include <exception>

#include "storage.h"
#include "../../util/base64.h"
#include "../../util/convert.h"

  namespace Binc {

  std::string label;
  std::string label_char;
  std::string section;
  std::string astring;
  std::string astring_sl;
  std::string astrings;
  std::string storage_erradd;
  std::string pair_first;

  std::map<std::string, std::string> aliastotext;
  std::map<std::string, std::string> texttoalias;

  extern Storage *internal__storage;

  /* external declarations */
  extern char *storageparsertext;
  extern std::string storage_quoted;
  int yylex();
  int yyerror(char *);

%}

%token LABELCHARS QUOTED HEX

%%

//------------------------------------------------------------------------
file		       : aliases sections
                       | sections
                       ;

//------------------------------------------------------------------------
aliases		       : aliases alias
		       | alias
		       ;

//------------------------------------------------------------------------
sections	       : sections section
		       | section
		       ;

//------------------------------------------------------------------------
alias                  : '?'
                         astring
{
  if (aliastotext.find(pair_first) != aliastotext.end())
    throw StorageException("alias already defined for " + pair_first);

  pair_first = astring;
  astring = "";
}
                         ':'
                         astring
{
  if (texttoalias.find(astring) != texttoalias.end() && astring != pair_first)
    throw StorageException("an alias already points to " + astring);
  
  aliastotext[pair_first] = astring;
  texttoalias[astring] = pair_first;
  
  astring = "";
  pair_first = "";
}
                         ';'
                       ;

//------------------------------------------------------------------------
section                : label
{
  if (aliastotext.find(label) != aliastotext.end())
    section = aliastotext[label];
  else
    section = label;
  
  label = "";
  storage_erradd = "expected '{'";
}
		         '{'
{
  storage_erradd = "expected comma seperated list of pairs";
}
			 pair_cl
{
  storage_erradd = "expected '}'";
}
			 '}'
{
  storage_erradd = "";
}
		       ;

//------------------------------------------------------------------------
label                  : LABELCHARS
{
  label = storageparsertext;
}
		       ;

//------------------------------------------------------------------------
pair_cl		       : pair_cl
{
  storage_erradd = "expected ','";
}
		         ','
{
  storage_erradd = "expected pair";
}
			 pair
		       | pair
		       ;

//------------------------------------------------------------------------
pair		       : astring_sl
{
  storage_erradd = "expected '='";
  
  if (aliastotext.find(astring_sl) != aliastotext.end())
    pair_first = aliastotext[astring_sl];
  else
    pair_first = astring_sl;
  astring_sl = "";
}
		         '='
{
  storage_erradd = "expected second string in a pair";
}
			 astring_sl
{
  internal__storage->setSection(section);
  internal__storage->insert(pair_first, astring_sl);
  astring_sl = "";
  pair_first = "";
}
		       ;

//------------------------------------------------------------------------
astring_sl             : astring_sl astring
{
  astring_sl += " ";
  astring_sl += astring;
  astring = "";
}
                       | astring
{
  astring_sl = astring;
  astring = "";
}
                       ;

//------------------------------------------------------------------------
astring		       : label
{
  astring = label;
  label = "";
}
                       | QUOTED
{
  astring = storage_quoted;
  storage_quoted = "";
}
                       | HEX
{
  std::string tmp = storageparsertext;
  if (tmp.size() > 2)
    tmp = tmp.substr(1, tmp.size() - 2);
  astring = Binc::fromHex(tmp);
}
		       ;

%%

#include "lex.storageparser.c"

}
